@@ -327,15 +327,6 @@ static gboolean DecodeHeader(unsigned char *BFH, unsigned char *BIH,
 		State->Header.n_colors = clrUsed;
 	else
             State->Header.n_colors = (1 << State->Header.depth);
-	
-	if (State->Header.n_colors > (1 << State->Header.depth)) {
-		g_set_error_literal (error,
-                                     GDK_PIXBUF_ERROR,
-                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                                     _("BMP image has bogus header data"));
-		State->read_state = READ_STATE_ERROR;
-		return FALSE;
-	}
 
 	State->Type = State->Header.depth;	/* This may be less trivial someday */
 
@@ -1417,7 +1408,7 @@ MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
 
 	info->name = "bmp";
 	info->signature = (GdkPixbufModulePattern *) signature;
-	info->description = N_("The BMP image format");
+	info->description = NC_("image format", "BMP");
 	info->mime_types = (gchar **) mime_types;
 	info->extensions = (gchar **) extensions;
 	info->flags = GDK_PIXBUF_FORMAT_WRITABLE | GDK_PIXBUF_FORMAT_THREADSAFE;
