@@ -301,7 +301,7 @@ MODULE_ENTRY (fill_info) (GdkPixbufFormat * info)
 
 	info->name = "jpeg2000";
 	info->signature = (GdkPixbufModulePattern *) signature;
-	info->description = N_("The JPEG 2000 image format");
+	info->description = NC_("image format", "JPEG 2000");
 	info->mime_types = (gchar **) mime_types;
 	info->extensions = (gchar **) extensions;
 	info->flags = GDK_PIXBUF_FORMAT_THREADSAFE;
