@@ -812,7 +812,7 @@ MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
 
 	info->name = "xpm";
 	info->signature = (GdkPixbufModulePattern *) signature;
-	info->description = N_("The XPM image format");
+	info->description = NC_("image format", "XPM");
 	info->mime_types = (gchar **) mime_types;
 	info->extensions = (gchar **) extensions;
 	info->flags = GDK_PIXBUF_FORMAT_THREADSAFE;
