@@ -26,32 +26,10 @@
 #define compare_option(p1, p2, key) \
   g_strcmp0 (gdk_pixbuf_get_option (p1, key), gdk_pixbuf_get_option (p2, key))
 
-static gboolean
-pixdata_equal (GdkPixbuf *p1, GdkPixbuf *p2)
-{
-  if (gdk_pixbuf_get_colorspace (p1) != gdk_pixbuf_get_colorspace (p2))
-    return FALSE;
-  if (gdk_pixbuf_get_n_channels (p1) != gdk_pixbuf_get_n_channels (p2))
-    return FALSE;
-  if (gdk_pixbuf_get_bits_per_sample (p1) != gdk_pixbuf_get_bits_per_sample (p2))
-    return FALSE;
-  if (gdk_pixbuf_get_width (p1) != gdk_pixbuf_get_width (p2))
-    return FALSE;
-  if (gdk_pixbuf_get_height (p1) != gdk_pixbuf_get_height (p2))
-    return FALSE;
-  if (gdk_pixbuf_get_rowstride (p1) != gdk_pixbuf_get_rowstride (p2))
-    return FALSE;
-  if (memcmp (gdk_pixbuf_get_pixels (p1), gdk_pixbuf_get_pixels (p2),
-          gdk_pixbuf_get_byte_length (p1)) != 0)
-    return FALSE;
-
-  return TRUE;
-}
-
 static gboolean
 pixbuf_equal (GdkPixbuf *p1, GdkPixbuf *p2)
 {
-  if (!pixdata_equal (p1, p2))
+  if (!pixdata_equal (p1, p2, NULL))
     return FALSE;
   if (compare_option (p1, p2, "Title") != 0)
     return FALSE;
@@ -97,6 +75,38 @@ test_save_roundtrip (void)
   g_object_unref (ref);
 }
 
+static void
+test_save_ico (void)
+{
+  GError *error = NULL;
+  GdkPixbuf *ref, *ref2;
+  GdkPixbuf *pixbuf;
+
+  if (!format_supported ("ico") || !format_supported ("png"))
+    {
+      g_test_skip ("format not supported");
+      return;
+    }
+
+  ref = gdk_pixbuf_new_from_file (g_test_get_filename (G_TEST_DIST, "test-image.png", NULL), &error);
+  g_assert_no_error (error);
+
+  ref2 = gdk_pixbuf_scale_simple (ref, 256, 256, GDK_INTERP_NEAREST);
+  g_object_unref (ref);
+  ref = ref2;
+
+  gdk_pixbuf_save (ref, "pixbuf-save-roundtrip", "ico", &error, NULL);
+  g_assert_no_error (error);
+
+  pixbuf = gdk_pixbuf_new_from_file ("pixbuf-save-roundtrip", &error);
+  g_assert_no_error (error);
+
+  g_assert (pixbuf_equal (pixbuf, ref));
+
+  g_object_unref (pixbuf);
+  g_object_unref (ref);
+}
+
 static void
 test_save_options (void)
 {
@@ -142,6 +152,7 @@ main (int argc, char **argv)
 
   g_test_add_func ("/pixbuf/save/roundtrip", test_save_roundtrip);
   g_test_add_func ("/pixbuf/save/options", test_save_options);
+  g_test_add_func ("/pixbuf/save/ico", test_save_ico);
 
   return g_test_run ();
 }
